<?php

// Exit if accessed directly
if ( !defined( 'ABSPATH' ) ) exit;

add_action('wp_footer', 	array('WPsite_Firebar_Core_Functions','display'));

// AJAX call to get all tags and custom posts

add_action( 'wp_ajax_wps_fb_set_close_cookie', 			array('WPsite_Firebar_Core_Functions', 'set_close_cookie'));
add_action( 'wp_ajax_nopriv_wps_fb_set_close_cookie', 	array('WPsite_Firebar_Core_Functions', 'set_close_cookie'));

/**
 *
 * Main class for all core functions
 *
 * @since 1.2.5
 */
class WPsite_Firebar_Core_Functions {

	/**
	 * Hook function to 'wp_footer' to display firebar on website
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function display(){

		include_once(WPSITE_FIREBAR_PLUGIN_DIR . "/admin/model/db.php");

		global $wpdb;
		global $post;
		global $cat;

		$query = "SELECT * FROM `" . WPsite_Firebar_DB::getTableName() . "`";
        $data = $wpdb->get_results($query,'ARRAY_A');
        $done = false;

        // Get all post types that are public

		$post_types = get_post_types(array('public' => true));

		// Post Types

		if (!$done && is_singular()) {

			foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$post_ids = explode(',', $trigger[$post->post_type]['id']);
        		$category_ids = explode(',', $trigger['category']['id']);

	    		// Check if post types match

	    		if (isset($trigger['type']) && $trigger['type'] != $post->post_type && $trigger['type'] != 'category') {
					continue;
		    	}

	    		// ID

        		if (isset($trigger[$post->post_type]['all']) &&
        			!$trigger[$post->post_type]['all'] &&
	    			in_array($post->ID, $post_ids) &&
	    			$val['active'] == 'Active' &&
	    			!$done) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }


			foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);
        		$post_ids = explode(',', $trigger[$post->post_type]['id']);
        		$category_ids = explode(',', $trigger['category']['id']);

	    		// Check if post types match

	    		if (isset($trigger['type']) && $trigger['type'] != $post->post_type && $trigger['type'] != 'category' && $trigger['type'] != 'tag') {
					continue;
		    	}

		    	// Tag ID

		    	$tags = explode(',', $trigger['tag']['id']);

        		if ($trigger['type'] == 'tag' && has_tag($tags,$post->ID) && $val['active'] == 'Active' && !$done) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

        		// Category ID

        		if ($trigger['type'] == 'category' &&
        			isset($trigger['category']['all']) &&
        			!$trigger['category']['all'] &&
	        		$val['active'] == 'Active' &&
        			in_category($category_ids, $post->ID) &&
        			!$done) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

        		// All

        		if ($trigger['type'] != 'category' &&
        			$trigger['type'] != 'tag' &&
        			isset($trigger[$post->post_type]['all']) &&
        			$trigger[$post->post_type]['all'] &&
        			$val['active'] == 'Active' &&
        			!$done) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
		}

        // Tag

        if (!$done && is_tag()) {
	        foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);
        		$tags = explode(',', $trigger['tag']['id']);

        		if ($trigger['type'] == 'tag' && is_tag($tags) && $val['active'] == 'Active' && !$done) {
	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
        }

		// Category

        if (!$done && (is_category() || is_single())) {
	        foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);
        		$category_ids = explode(',', $trigger['category']['id']);

        		if ($trigger['type'] != 'category' || $val['active'] != 'Active' || $done) {
	        		continue;
        		}

        		// ID

        		if (isset($trigger['category']['all']) &&
	        		!$trigger['category']['all'] &&
        			(is_category() && in_array($cat, $category_ids)) ||
        			(is_singular() && in_category($category_ids, $post->ID))) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	       }

	       foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);
        		$category_ids = explode(',', $trigger['category']['id']);

        		if ($trigger['type'] != 'category' || $val['active'] != 'Active' || $done) {
	        		continue;
        		}

        		// All

        		if (isset($trigger['category']['all']) &&
        			$trigger['category']['all'] &&
        			is_category()) {

	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
        }

        // Archive

        if (!$done && is_archive()) {
	        foreach ($data as $val) {

        		$args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);

        		if ($trigger['type'] == 'archive' && $val['active'] == 'Active' && !$done) {
	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
        }

        // Front Page

        if (!$done && is_front_page()) {
	        foreach ($data as $val) {

		        $args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);

        		if ($trigger['type'] == 'frontpage' && $val['active'] == 'Active' && !$done) {
	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
        }

        // Site

        if (!$done) {
	        foreach ($data as $val) {

		        $args = self::mb_unserialize($val['args']);
        		$trigger = self::mb_unserialize($val['trigger']);

		        if (!self::check_conditions($val['id'], $val['template'], $args, $trigger)) {
			        continue;
		        }

	    		$social = self::mb_unserialize($val['social']);
        		$news_letter = self::mb_unserialize($val['news_letter']);
        		$trigger = self::mb_unserialize($val['trigger']);

        		if ($trigger['type'] == 'site' && $val['active'] == 'Active' && !$done) {
	        		self::generate_code($val, $social, $news_letter);
	        		$done = true;
					break;
        		}

	        }
        }
	}

	/**
	 * Checks that all fire slide-in conditions are met
	 *
	 * @access public
	 * @static
	 * @param mixed $firebar_id
	 * @param mixed $template
	 * @param mixed $args
	 * @param mixed $trigger
	 * @return void
	 */
	static function check_conditions($firebar_id, $template, $args, $trigger) {

		global $post;

		// Check if user has already signed up

		if (strtolower(str_replace(' ', '-', $template)) == 'news-letter' && isset($_COOKIE[WPsite_Firebar::$prefix . 'email_success_' . $firebar_id])) {
			return false;
		}

		// Check if user closed popup and dont not show is set to true

		if ( isset($args['close_button']) && $args['close_button'] &&
			isset($args['dont_show_again']) && $args['dont_show_again'] &&isset($_COOKIE[WPsite_Firebar::$prefix . 'close_' . $firebar_id])) {
			return false;
		}

		// Check user role

		if (!self::check_user_role($args)) {
    		return false;
		}

		// Check referrer

		if (!self::check_referrer($args)) {
			return false;
		}

		// Check if on mobile

		if (isset($args['mobile']) && $args['mobile'] == 'no' && wp_is_mobile()) {
    		return false;
		}

		// Check for excluded posts

		if ( is_singular() &&
			 isset($trigger[$post->post_type]['exclude']) &&
			 in_array($post->ID, explode(',', $trigger[$post->post_type]['exclude'])) ) {

    		return false;
		}

		// Date Range

		$now = time();

		if (isset($args['start_date']) && isset($args['end_date']) &&
			(strtotime($args['start_date']) > $now || strtotime($args['end_date']) < $now)) {
    		return false;
		}

		return true;
	}

	/**
     * Check if the current user has access to backend / frontend based on his role compared with role from settings
     *
     * @access public
     * @static
     * @param mixed $settings
     * @return void
     */
    static function check_user_role($settings) {

	    // If user has not set this option yet then display FireBar to everyone

	    if (!isset($settings['users'])) {
		    return true;
	    }

		// Check if Everyone users is true

		if (isset($settings['users']) && $settings['users'] == 'everyone') {
			return true;
		}

		// Check if All users is true

		else if (isset($settings['users']) && $settings['users'] == 'logged_in' && is_user_logged_in()) {
			return true;
		}

		// Check if Logged Out Users is true

		else if (isset($settings['users']) && $settings['users'] == 'logged_out' && !is_user_logged_in()) {
			return true;
		}

		// Check if Specific User is true

		else if (isset($settings['users']) && $settings['users'] == 'specific' && is_user_logged_in()) {

			$is_allowed = false;
			global $wp_roles;
			$current_user = wp_get_current_user();

			foreach ($wp_roles->role_names as $role_name => $role) {

				if (isset($settings['user_roles'][$role_name]) && $settings['user_roles'][$role_name] && in_array($role_name, $current_user->roles)) {
					$is_allowed = true;
				}
			}

			return $is_allowed;
		}

        return false;
    }

    /**
	 * Determines if the referrer URL matches any given by the user
	 *
	 * @access public
	 * @static
	 * @param mixed $settings
	 * @return void
	 */
	static function check_referrer($settings) {

		// If the use has not set a referrer type yet then just display the FireBar

		if (!isset($settings['referrer_type'])) {
			return true;
		}

		// Return true any domain is checked

		if ($settings['referrer_type'] == 'any') {
			return true;
		}

		// Check is referrer domain matches ones given by user

		else if (isset($_SERVER["HTTP_REFERER"])) {

			$urls = explode(',', $settings['referrer']);
			$domain = parse_url($_SERVER["HTTP_REFERER"]);

			if (in_array($domain['host'], $urls)) {
				return true;
			}
		}

		return false;
	}

	/**
	 * Adds the scripts and localizes the variables
	 *
	 * @since 1.0.0
	 *
	 * @param 	$val array of attrs
	 * @return 	N/A
	 */
	static function generate_scripts($val) {

		include_once(WPSITE_FIREBAR_PLUGIN_DIR . "/admin/functions.php");

		$default_firebar = WPsite_Firebar_Admin_Functions::default_firebar();

		$news_letter = self::mb_unserialize($val['news_letter']);
	    $args = self::mb_unserialize($val['args']);
	    $text_link = self::mb_unserialize($val['text_link']);

		wp_enqueue_style(WPsite_Firebar::$prefix . 'glyphicons_css', WPSITE_FIREBAR_PLUGIN_URL . '/admin/css/glyphicons.css');
		wp_enqueue_style(WPsite_Firebar::$prefix . 'fontawesome_css', WPSITE_FIREBAR_PLUGIN_URL . '/admin/fonts/font-awesome.min.css');

		if ($val['template'] == 'Recent Posts' || $val['template'] == 'Twitter Feed') {
			wp_enqueue_script(WPsite_Firebar::$prefix . 'newsticker_js', WPSITE_FIREBAR_PLUGIN_URL . '/core/js/newsTicker.js');
		}

		// Google Font Loader

		if (isset($args['google_fonts']) && $args['google_fonts']) {
			wp_enqueue_script(WPsite_Firebar::$prefix . 'google_font_js', WPSITE_FIREBAR_PLUGIN_URL .  '/core/js/webfontloader.js');
		}

	    $args_array = array(
	    	'prefix_dash'		=> WPsite_Firebar::$prefix_dash,
	    	'ajaxurl'			=> admin_url( 'admin-ajax.php' ),
	    	'location' 			=> isset($val['location']) 					? esc_attr($val['location']) : $default_firebar['location'],
	    	'icon_color' 		=> isset($args['icon_color']) 				? esc_attr($args['icon_color']) : $default_firebar['args']['icon_color'],
	    	'border_color' 		=> isset($args['border_color']) 			? esc_attr($args['border_color']) : $default_firebar['args']['border_color'],
	    	'border_size' 		=> isset($args['border_size']) 				? esc_attr($args['border_size']) : $default_firebar['args']['border_size'],
	    	'position' 			=> isset($args['position']) 				? esc_attr($args['position']) : $default_firebar['args']['position'],
	    	'effect' 			=> isset($args['effect']) 					? esc_attr($args['effect']) : $default_firebar['args']['effect'],
	    	'color' 			=> isset($val['color']) 					? esc_attr($val['color']) : $default_firebar['color'],
	    	'text_color' 		=> isset($val['text_color']) 				? esc_attr($val['text_color']) : $default_firebar['text_color'],
	    	'display' 			=> isset($val['display']) 					? esc_attr($val['display']) : $default_firebar['display'],
	    	'height' 			=> isset($val['height']) 					? esc_attr($val['height']) : $default_firebar['height'],
	    	'animation' 		=> isset($val['animation']) 				? esc_attr($val['animation']) : $default_firebar['animation'],
	    	'move_body' 		=> isset($_GET['action']) 					&& $_GET['action'] == 'preview' ? false : (isset($args['move_body']) ?
	    																	esc_attr($args['move_body']) : $default_firebar['args']['move_body']),
	    	'font_size' 		=> isset($args['font_size']) 				? esc_attr($args['font_size']) : $default_firebar['args']['font_size'],
	    	'font_weight' 		=> isset($args['font_weight']) 				? esc_attr($args['font_weight']) : $default_firebar['args']['font_weight'],
	    	'font_family' 		=> isset($args['font_family']) 				? esc_attr($args['font_family']) : $default_firebar['args']['font_family'],
	    	'google_fonts' 		=> isset($args['google_fonts']) 			? esc_attr($args['google_fonts']) : $default_firebar['args']['google_fonts'],
	    	'link_text_color' 	=> isset($text_link['link_text_color']) 	? esc_attr($text_link['link_text_color']) : $default_firebar['text_link']['link_text_color'],
	    	'link_button_color' => isset($text_link['link_button_color']) 	? esc_attr($text_link['link_button_color']) : $default_firebar['text_link']['link_button_color'],
	    	'nl_text_color' 	=> isset($args['nl_text_color']) 			? esc_attr($args['nl_text_color']) : $default_firebar['args']['nl_text_color'],
	    	'nl_button_color' 	=> isset($args['nl_button_color']) 			? esc_attr($args['nl_button_color']) : $default_firebar['args']['nl_button_color'],
	    	'close_button'		=> isset($args['close_button']) 			&& $args['close_button'] ? true : false,
	    	'dont_show_again'	=> isset($args['dont_show_again']) 			&& $args['dont_show_again'] ? true : false,
	    );

	    // Load all files necessary for the Preview

	    if (isset($_GET['action']) && $_GET['action'] == 'preview') {

			wp_enqueue_style(WPsite_Firebar::$prefix . strtolower(str_replace(' ', '-', $val['template'])) . 'preview_css', WPSITE_FIREBAR_PLUGIN_URL . '/core/css/' . strtolower(str_replace(' ', '-', $val['template'])) . '.css');
			wp_enqueue_style(WPsite_Firebar::$prefix . 'preview_css', WPSITE_FIREBAR_PLUGIN_URL . '/core/css/firebar.css');


	    	wp_enqueue_script(WPsite_Firebar::$prefix . 'preview_js', WPSITE_FIREBAR_PLUGIN_URL . '/core/js/firebar.js', array(
		    	'jquery',
		    	'jquery-effects-' . $args['effect'],
		    ));

		    wp_localize_script(WPsite_Firebar::$prefix . 'preview_js', 'wps_fb_data', $args_array);

	    }

	    // Load all file for the real Firebar

	    else {

			wp_enqueue_style(WPsite_Firebar::$prefix . strtolower(str_replace(' ', '-', $val['template'])) . 'css', WPSITE_FIREBAR_PLUGIN_URL . '/core/css/' . strtolower(str_replace(' ', '-', $val['template'])) . '.css');
		    wp_enqueue_style(WPsite_Firebar::$prefix . 'css', WPSITE_FIREBAR_PLUGIN_URL . '/core/css/firebar.css');


		    wp_enqueue_script(WPsite_Firebar::$prefix . 'js', WPSITE_FIREBAR_PLUGIN_URL . '/core/js/firebar.js', array(
		    	'jquery',
		    	'jquery-effects-' . $args['effect'],
		    ));

		    wp_localize_script(WPsite_Firebar::$prefix . 'js', 'wps_fb_data', $args_array);
	    }

	}

	/**
	 * Displays the firebar
	 *
	 * @since 1.0.0
	 *
	 * @param 	$val the array of the firebar attr
	 * @param 	$social the array of the firebar attr
	 * @param 	$news letter the array of the firebar attr
	 * @return 	N/A
	 */
	static function generate_code($val, $social, $news_letter) {

		// Include scripts

		self::generate_scripts($val);

		$args                     = self::mb_unserialize($val['args']);
		$text_link                = self::mb_unserialize($val['text_link']);
		$recent_posts             = $args['recent_posts'];
		$twitter                  = $args['twitter'];
		$location                 = isset($val['location']) ? $val['location'] : 'top';
		$stub_location            = isset($args['stub_location']) ? $args['stub_location'] : 'right';
		$template                 = isset($val['template']) ? $val['template'] : '';
		$height                   = isset($val['height']) ? esc_attr($val['height']) : 40;
		$title                    = isset($val['title']) ? $val['title'] : '';
		$link                     = isset($val['link_url']) ? $val['link_url'] : '';
		$link_name                = isset($val['link_name']) ? $val['link_name'] : '';
		$mailchimp_api_key        = isset($news_letter['MailChimp']['api_key']) ? $news_letter['MailChimp']['api_key'] : '';
		$mailchimp_list_id        = isset($news_letter['MailChimp']['list_id']) ? $news_letter['MailChimp']['list_id'] : '';
		$aweber_consumer_key      = isset($news_letter['aweber']['consumer_key']) ? $news_letter['aweber']['consumer_key'] : '';
		$aweber_consumer_secret   = isset($news_letter['aweber']['consumer_secret']) ? $news_letter['aweber']['consumer_secret'] : '';
		$aweber_access_key        = isset($news_letter['aweber']['access_key']) ? $news_letter['aweber']['access_key'] : '';
		$aweber_access_secret     = isset($news_letter['aweber']['access_secret']) ? $news_letter['aweber']['access_secret'] : '';
		$aweber_account_id        = isset($news_letter['aweber']['account_id']) ? $news_letter['aweber']['account_id'] : '';
		$aweber_list_id           = isset($news_letter['aweber']['list_id']) ? $news_letter['aweber']['list_id'] : '';
		$feedburner_id            = isset($news_letter['feedburner']['id']) ? $news_letter['feedburner']['id'] : '';
		$news_letter_type         = isset($news_letter['type']) ? $news_letter['type'] : '';
		$custom                   = isset($val['custom']) ? $val['custom'] : '';

		// Display data based off if location (Top or Bottom) and template

		include_once(WPSITE_FIREBAR_PLUGIN_DIR . 'core/views/display-firebar.php');
	}

	/**
	 * Gets all the class names for the Firebar
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_classes($args = null, $template = null, $location = null, $stub_location = null){

		// Return nothing if we did not get our data

		if ( !isset($args) || !isset($template) || !isset($location) || !isset($stub_location)) {
			return null;
		}

		$shadow       = isset($args['shadow']) && $args['shadow'] == 'yes' ?
						WPsite_Firebar::$prefix_dash . 'container-' . strtolower($location) . '-shadow' :
						'';

		$container    = WPsite_Firebar::$prefix_dash . 'container ' .
						WPsite_Firebar::$prefix_dash . 'container-' . strtolower($location) . ' ' . $shadow . ' ' .
						WPsite_Firebar::$prefix_dash . 'container-' . strtolower(str_replace(' ', '-', $template));

		$stub         = WPsite_Firebar::$prefix_dash . 'stub ' .
						WPsite_Firebar::$prefix_dash . 'stub-' . strtolower($location) . ' ' .
						WPsite_Firebar::$prefix_dash . 'stub-' . $stub_location;

		$close        = WPsite_Firebar::$prefix_dash . 'close ' .
						WPsite_Firebar::$prefix_dash . 'close-' . $stub_location . ' fa ' . $args['icon'];


		return array(
			'container'	=> $container,
			'stub'		=> $stub,
			'close'		=> $close,
		);
	}

	/**
     * Sets the close cookie so that the optin does not show after user closes it out
     *
     * @access public
     * @static
     * @return void
     */
    static function set_close_cookie() {

	    if (!isset($_POST['firebar_id'])) {
		    echo 'FireBar ID was not found so cookie could not be set.';
		    die();
	    }

	    setcookie(WPsite_Firebar::$prefix . 'close_' . $_POST['firebar_id'], true, 0, '/');

		echo 'Cookie was set';
		die();
    }

	/**
	 * Mulit-byte Unserialize
	 *
	 * UTF-8 will screw up a serialized string
	 *
	 * @access private
	 * @param string
	 * @return string
	 */
	static function mb_unserialize($string) {
	    $string2 = preg_replace_callback(
	        '!s:(\d+):"(.*?)";!s',
	        function($m){
	            $len = strlen($m[2]);
	            $result = "s:$len:\"{$m[2]}\";";
	            return $result;

	        },
	        $string);
	    return unserialize($string2);
	}

}